﻿using System;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using Sat2Ip.Library.Ts;

namespace Sat2Ip.Library.Rtp
{
    public class RtpClient
    {

        private static int bufferlenght = 128*1316;
        byte[] data = new byte[bufferlenght];
        
        private EndPoint _endPoint;
        
        private bool writeHeaderToConsole = true;
        private bool listening = false;
        private int _port;
        private string _address;
        private Thread listenerThread;
        private Socket _sock;
        private int _rcvd;
       
        private FileStream _fsdata;
        private RtpPacket _packet;
        private FileStream _fspayload;
        private MemoryStream _timeshift;
        /// <summary>
        /// Returns a reference to the audio stream
        /// </summary>
        //public SpeechStreamer AudioStream
        //{
        //    get { return audioStream; }
        //}

        /// <summary>
        /// Gets whether the client is listening for packets
        /// </summary>
        public bool Listening
        {
            get { return listening; }
        }

        /// <summary>
        /// Gets the port the RTP client is listening on
        /// </summary>
        public int Port
        {
            get { return _port; }
        }

        public MemoryStream Timeshift
        {
            get { return _timeshift; }
            set { _timeshift = value; }
        }


        /// <summary>
        /// RTP Client for receiving an RTP stream containing a WAVE audio stream
        /// </summary>
        /// <param name="port">The port to listen on</param>
        public RtpClient(string address,int port)
        {
            Console.WriteLine(" [RTPClient] Loading...");

            _port = port;
            _address = address;
           


            Console.WriteLine(" Done");
        }

        /// <summary>
        /// Creates a connection to the RTP stream
        /// </summary>
        public void StartClient(string path,string name)
        {
            var pathname = string.Format(@"{0}:\{1}.ts", path, name);

            _timeshift = new MemoryStream();
            _fspayload = new FileStream(@pathname, FileMode.Create, FileAccess.ReadWrite, FileShare.ReadWrite, 1048576, true);
            var rtpEndPoint = new IPEndPoint(IPAddress.Parse(_address), _port);
            var endPoint =(EndPoint) rtpEndPoint;
            _sock = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
            _sock.Bind(endPoint);
            
            var sender = new IPEndPoint(IPAddress.Parse(_address), _port);
            _endPoint = (EndPoint)sender;
            //
            listening = true;
            listenerThread = new Thread(ReceiveCallback);
            listenerThread.Start();

            Console.WriteLine(" [RTPClient] Listening for packets on port " + _port + "...");
        }

        /// <summary>
        /// Tells the UDP client to stop listening for packets.
        /// </summary>
        public void StopClient()
        {
            // Set the boolean to false to stop the asynchronous packet receiving
            listening = false;
            Console.WriteLine(" [RTPClient] Stopped listening on port " + _port);
        }

        /// <summary>
        /// Handles the receiving of UDP packets from the RTP stream
        /// </summary>
        /// <param name="ar">Contains packet data</param>
        private void ReceiveCallback()
        {
            // Begin looking for the next packet

            while (listening)
            {
                _rcvd = _sock.ReceiveFrom(data, ref _endPoint);
                Console.WriteLine(_rcvd);
                _packet = new RtpPacket(data, _rcvd);

                _timeshift.BeginWrite(_packet.Payload, 0, _packet.GetPayloadLength(), null, null);
                //WriteObject wo = new WriteObject();
                //wo.FStream = _fspayload;
                //IAsyncResult asyncResult;
                //_fspayload.BeginWrite(_packet.Payload, 0, _packet.GetPayloadLength(), EndWriteCallBack, wo);
                
            }

        }
        private void EndWriteCallBack(IAsyncResult res)
        {
            var wo = (WriteObject)res.AsyncState;
            var ftemp = wo.FStream;
            ftemp.EndWrite(res);
        }

        /// <summary>
        /// Grabs a value from the RTP header in Big-Endian format
        /// </summary>
        /// <param name="packet">The RTP packet</param>
        /// <param name="startBit">Start bit of the data value</param>
        /// <param name="endBit">End bit of the data value</param>
        /// <returns>The value</returns>
        private int GetRtpHeaderValue(byte[] packet, int startBit, int endBit)
        {
            int result = 0;

            // Number of bits in value
            int length = endBit - startBit + 1;

            // Values in RTP header are big endian, so need to do these conversions
            for (int i = startBit; i <= endBit; i++)
            {
                int byteIndex = i/8;
                int bitShift = 7 - (i%8);
                result += ((packet[byteIndex] >> bitShift) & 1)*(int) Math.Pow(2, length - i + startBit - 1);
            }
            return result;
        }
    }
    internal class WriteObject
    {
        FileStream fStream;

        public FileStream FStream
        {
            get { return fStream; }
            set { fStream = value; }
        }
    }
}
