﻿
﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.Net.Sockets;
 
namespace Sat2Ip.Library.Rtp
{
        /// <summary>
      /// MCIPAddress Class
       /// </summary>
      public class MCIPAddress
      {
           // Überprüfe ob es sich um eine gültige IPv4 Multicast-Adresse handelt
          public static bool isValid(string ip)
          {
              try
              {
                   int octet1 = Int32.Parse(ip.Split(new Char[] { '.' }, 4)[0]);
                   if ((octet1 >= 224) && (octet1 <= 239))
                      return true;
             }
              catch (Exception ex)
              {
                 string str = ex.Message;
              }
 
              return false;
          }
      }

      /// <summary>
      /// Multicast Receiver Klasse
      /// </summary>
      public class RtpReciever
      {
          /// <summary>
         /// Konstruktor
         /// </summary>
          public RtpReciever(int bufferSize)
          {
              bytes = new Byte[bufferSize];
          }
   
          //Attribute
          Socket _socket;
          IPAddress _address;
          Int32 _port;
          EndPoint _endPoint;
          readonly Byte[] bytes;
          bool _isConnected = false;
          Object _locker = new Object();
          private bool ismulticast;
  
          //Delegates bzw. Events
         public delegate void DelegateDataReceived(Byte[] bytes);
         public delegate void DelegateDisconnected(string Reason);
         public delegate void DelegateExceptionAppeared(Exception ex);
         public event DelegateDataReceived DataReceived;
         public event DelegateDisconnected Disconnected;
  
          /// <summary>
          /// Connected
          /// </summary>
          public bool Connected
          {
             get
              {
                  return _isConnected;
              }
          }
          /// <summary>
          /// Connect
          /// </summary>
          /// <param name="strAdress"></param>
          /// <param name="port"></param>
          public void Connect(string strAddress, int port)
          {
              _address = IPAddress.Parse(strAddress);
              _port = port;
              ismulticast = MCIPAddress.isValid(strAddress);
              if(!ismulticast)
              {
                  //Unicast Socket erstellen
                  _socket = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
                  //Adresse wiederverwenden
                  _socket.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, 1);
                  //Generiere Endpunkt
                  var rtpEndPoint = new IPEndPoint(IPAddress.Parse(strAddress), _port);
                  var endPoint = (EndPoint) rtpEndPoint;
                  _socket.Bind(endPoint);
                  var sender = new IPEndPoint(IPAddress.Parse(strAddress), _port);
                  _endPoint = (EndPoint) sender;
                  _isConnected = true;
                  this.DoRead();
              }
              else
              {
                  //Socket erstellen
                  _socket = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp); // Multicast Socket
                  //Adresse wiederverwenden
                  _socket.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, 1);
                  //Generiere Endpunkt
                  _endPoint = new IPEndPoint(IPAddress.Any, _port);
                  _socket.Bind(_endPoint);
                  //Mitgliedschaft in der Multicast Gruppe bekannt geben
                  _socket.SetSocketOption(SocketOptionLevel.IP, SocketOptionName.AddMembership, new MulticastOption(_address, IPAddress.Any));
                  //Verbunden
                  _isConnected = true;
                  this.DoRead();
              }
              
           }
          /// <summary>
          /// Startet das Lesen 
          /// </summary>
          /// <param name="Data"></param>
          private void DoRead()
          {
              try
              {
                  _socket.BeginReceive(bytes, 0, bytes.Length, SocketFlags.None, new AsyncCallback(OnDataReceived), _socket);
              }
              catch (Exception ex)
              {
                  throw new Exception(ex.Message);
              }
          }
          /// <summary>
          /// OnDataReceived
          /// </summary>
          /// <param name="ar"></param>
          private void OnDataReceived(IAsyncResult ar)
          {
             try
              {
                  //So lange wie verbunden
                  if (_isConnected)
                  {
                      //Anzahl gelesener Bytes
                      int read = _socket.EndReceive(ar);
  
                     //Wenn Daten vorhanden
                      if (read > 0)
                      {
                          //Wenn das Event verwendet wird
                          if (this.DataReceived != null)
                         {
                              //Wenn gelesene Bytes und Datengrösse übereinstimmen
                              if (read == bytes.Length)
                             {
                                 //Event abschicken
                                  this.DataReceived(bytes);
                             }
                              else
                            {
                                 //Nur den gelesenen Teil übermitteln
                                 Byte[] readed = new Byte[read];
                                  Array.Copy(bytes, readed, read);
  
                                  //Event abschicken
                                 this.DataReceived(readed);
                             }
                         }
  
                         //Weiterlesen
                          _socket.BeginReceive(bytes, 0, bytes.Length, SocketFlags.None, new AsyncCallback(OnDataReceived), null);
                     }
                      else
                      {
                          //Verbindung beendet
                          _socket.Close();
                      }
                  }
              }
              catch (Exception ex)
              {
                  System.Diagnostics.Debug.WriteLine(String.Format("MulticastReceiver.cs | OnDataReceived() | {0}", ex.Message));
                  //Verbindung beenden
                  Disconnect();
             }
         }
          /// <summary>
          /// Disconnect
          /// </summary>
          public void Disconnect()
          {
              if (_isConnected)
             {
                 //Nicht mehr verbunden
                 _isConnected = false;
                  //Kündige Mitgliedschaft in der Multicast Gruppe
                 if (ismulticast)
                 {
                     _socket.SetSocketOption(SocketOptionLevel.IP, SocketOptionName.DropMembership, new MulticastOption(_address, IPAddress.Any));
                 }
                  //Verbindung beenden
                 _socket.Close();
  
                  //Event abschicken
                  if (this.Disconnected != null)
                  {
                     this.Disconnected("Connection has been finished");
                  }
              }
          }
      }
  }

 



      

 
