﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;

namespace Sat2Ip.Library.Rtp
{
    public class RtpStream : Stream
    {
        private AutoResetEvent _writeEvent;
        private List<byte> _buffer;
        private int _bufferSize;
        private int _readPosition;
        private int _writePosition;
        private bool _reset;

        public RtpStream(int bufferSize)
        {
            _writeEvent = new AutoResetEvent(false);
             _bufferSize = bufferSize;
             _buffer = new List<byte>(_bufferSize);
             for (int i = 0; i < _bufferSize;i++ )
                 _buffer.Add(new byte());
            _readPosition = 0;
            _writePosition = 0;
        }


        public override void Flush()
        {
            
        }

        public override bool CanRead
        {
            get { return true; }
        }

        public override bool CanSeek
        {
            get { return false; }
        }

        public override bool CanWrite
        {
            get { return true; }
        }

        public override long Length
        {
            get { return -1L; }
        }

        public override long Position
        {
            get { return 0L; }
            set { }
        }

        public override long Seek(long offset, SeekOrigin origin)
        {
            return 0L;
        }

        public override void SetLength(long value)
        {

        }

        public override int Read(byte[] buffer, int offset, int count)
        {
            int i = 0;
            while (i < count && _writeEvent != null)
            {
                if (!_reset && _readPosition >= _writePosition)
                {
                    _writeEvent.WaitOne(100, true);
                    continue;
                }
                buffer[i] = _buffer[_readPosition + offset];
                _readPosition++;
                if (_readPosition == _bufferSize)
                {
                    _readPosition = 0;
                    _reset = false;
                }
                i++;
            }

            return count;
        }

        public override void Write(byte[] buffer, int offset, int count)
        {
            for (int i = offset; i < offset + count; i++)
            {
                _buffer[_writePosition] = buffer[i];
                _writePosition++;
                if (_writePosition == _bufferSize)
                {
                    _writePosition = 0;
                    _reset = true;
                }
            }
            _writeEvent.Set();

        }

        public override void Close()
        {
            _writeEvent.Close();
            _writeEvent = null;
            base.Close();
        }


    }
}
