﻿using System;
using System.ComponentModel;
using System.Globalization;
using System.Text.RegularExpressions;
using UPNPLib;

namespace Sat2Ip.Library.Rtsp
{
    public class RtspDevice : INotifyPropertyChanged, IDisposable
    {
        #region Private Fields

        private String _rtspServerAddress;
        private String _rtspServerPort;
        private String _rtpClientPort;
        private String _rtcpClientPort;
        private String _rtspDeviceName;
        private Int32 _rtp;
        private RtspDeviceType _rtspDeviceType;
        private RtspSession _rtspSession;

        #endregion

        #region Constructor

        public RtspDevice(string devicename, string address, string port, string rtp, string rtcp)
        {
            _rtspDeviceName = devicename;
            _rtspServerAddress = address;
            _rtspServerPort = port;
            _rtpClientPort = rtp;
            _rtcpClientPort = rtcp;

        }

        public RtspDevice(UPnPDevice device)
        {
            var addressline = Regex.Split(device.PresentationURL, @"://+");
            var address = addressline[1].Split('/');
            _rtspDeviceName = device.FriendlyName;
            _rtspServerAddress = address[0];
            _rtspSession = new RtspSession(this);
        }

        #endregion

        #region Properties

        public String RtspServerAddress
        {
            get { return _rtspServerAddress; }
            set { if (_rtspServerAddress != value){_rtspServerAddress = value;OnPropertyChanged("RtspServerAddress");}}
        }

        public String RtspServerPort
        {
            get { if (string.IsNullOrEmpty(_rtspServerPort)){_rtspServerPort = "554";}return _rtspServerPort;}
            set { if (_rtspServerPort != value){_rtspServerPort = value;OnPropertyChanged("RtspServerPort");}}
        }

        public String RtpClientPort
        {
            get { if(string.IsNullOrEmpty(_rtpClientPort)){_rtp = Utils.GenerateRandomNumber(52000, 53000);_rtpClientPort = _rtp.ToString(CultureInfo.InvariantCulture);}return _rtpClientPort; }
            set { if (_rtpClientPort != value) { _rtpClientPort = value; OnPropertyChanged("RtpClientPort"); } }
        }

        public String RtcpClientPort
        {
            get{if (string.IsNullOrEmpty(_rtcpClientPort)){ var rtcp =_rtp + 1; _rtcpClientPort = rtcp.ToString(CultureInfo.InvariantCulture); } return _rtcpClientPort; }
            set { if (_rtcpClientPort != value){_rtcpClientPort = value;OnPropertyChanged("RtcpClientPort");}}
        }

        public String RtspDeviceName
        {
            get { return _rtspDeviceName; }
            set { if (_rtspDeviceName != value) { _rtspDeviceName = value; OnPropertyChanged("RtspDeviceName"); } }
        }

        public RtspSession RtspSession
        {
            get { return _rtspSession; }
            set{if (_rtspSession != value){_rtspSession = value; OnPropertyChanged("RtspSession"); } }
        }

        public RtspDeviceType RtspDeviceType
        {
            get { return _rtspDeviceType; }
            set{ if (_rtspDeviceType != value) { _rtspDeviceType = value; OnPropertyChanged("RtspDeviceType"); } }
        }

        #endregion

        #region Public Events

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        #region Protected Methods

        protected void OnPropertyChanged(string name)
        {
            var handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(name));
            }
        }

        #endregion

        public void Dispose()
        {
            if (_rtspSession != null)
            {
                _rtspSession.Dispose();
            }
        }
    }
}
